/**
 * Licensed to Abiquo Holdings S.L. (Abiquo) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package com.abiquo.server.core.cloud;

import javax.validation.constraints.NotNull;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

import com.abiquo.model.enumerator.HypervisorType;
import com.abiquo.model.transport.SingleResourceWithLimitsDto;
import com.abiquo.server.core.infrastructure.network.VLANNetworkDto;

/**
 * The Class VirtualDatacenterDto.
 * 
 * @restlink <link rel="datacenter" href="http://example.com/api/admin/datacenters/{datacenterId}"/>
 * @restlink <link rel="enterprise" href="http://example.com/api/admin/enterprises/{enterpriseId}"/>
 */
@XmlRootElement(name = "virtualDatacenter")
@XmlType(propOrder = {"id", "hypervisorType", "name", "vlan"})
public class VirtualDatacenterDto extends SingleResourceWithLimitsDto
{

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = -2165018992377526633L;

    /** The Constant BASE_MEDIA_TYPE. */
    public static final String BASE_MEDIA_TYPE = "application/vnd.abiquo.virtualdatacenter+xml";

    /** The Constant MEDIA_TYPE. */
    public static final String MEDIA_TYPE = BASE_MEDIA_TYPE + "; version=" + API_VERSION;

    /** The id. */
    private Integer id;

    /** The name. */
    private String name;

    /** The hypervisor type. */
    private HypervisorType hypervisorType;

    /** The vlan network for this virtual datacenter. */
    private VLANNetworkDto vlan;

    /*
     * (non-Javadoc)
     * @see com.abiquo.model.transport.SingleResourceTransportDto#getBaseMediaType()
     */
    @Override
    public String getBaseMediaType()
    {
        return BASE_MEDIA_TYPE;
    }

    /**
     * Hypervisor type of the virtual datacenter. Virtualization technology supported for this
     * virtual datacenter.
     * 
     * @return the hypervisor type
     */
    @NotNull
    public HypervisorType getHypervisorType()
    {
        return hypervisorType;
    }

    /**
     * Opaque ID of the virtual datacenter.
     * 
     * @output
     * @return the id
     */
    public Integer getId()
    {
        return id;
    }

    /*
     * (non-Javadoc)
     * @see com.abiquo.model.transport.SingleResourceTransportDto#getMediaType()
     */
    @Override
    public String getMediaType()
    {
        return VirtualDatacenterDto.MEDIA_TYPE;
    }

    /**
     * Virtual datacenter's name.
     * 
     * @return the name
     */
    @NotNull
    public String getName()
    {
        return name;
    }

    /**
     * Virtual datacenter network.
     * 
     * @return the vlan
     */
    @NotNull
    @XmlElement(name = "network")
    public VLANNetworkDto getVlan()
    {
        return vlan;
    }

    /**
     * Sets the hypervisor type.
     * 
     * @param hypervisorType the new hypervisor type
     */
    public void setHypervisorType(final HypervisorType hypervisorType)
    {
        this.hypervisorType = hypervisorType;
    }

    /**
     * Sets the id.
     * 
     * @param id the new id
     */
    public void setId(final Integer id)
    {
        this.id = id;
    }

    /**
     * Sets the name.
     * 
     * @param name the new name
     */
    public void setName(final String name)
    {
        this.name = name;
    }

    /**
     * Sets the vlan.
     * 
     * @param vlan the new vlan
     */
    public void setVlan(final VLANNetworkDto vlan)
    {
        this.vlan = vlan;
    }
}
