/**
 * Licensed to Abiquo Holdings S.L. (Abiquo) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package com.abiquo.server.core.cloud;

import java.util.Iterator;
import java.util.List;

import javax.validation.constraints.NotNull;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

import org.apache.commons.lang.StringUtils;

import com.abiquo.model.rest.RESTLink;
import com.abiquo.model.transport.SingleResourceTransportDto;
import com.abiquo.server.core.cloud.chef.RunlistElementsDto;
import com.abiquo.server.core.infrastructure.storage.DvdManagementDto;

/**
 * @restlink <link rel="template" href=
 *           "example.com/api/admin/enterprises/{id}/datacenterrepositories/{id}/virtualmachinetemplates/{id}"
 *           />
 */
@XmlRootElement(name = "virtualMachine")
@XmlType(propOrder = {"id", "uuid", "name", "description", "cpu", "ram", "hdInBytes", "vdrpPort",
"vdrpIP", "idState", "state", "idType", "highDisponibility", "password", "keymap", "dvd", "runlist"})
public class VirtualMachineDto extends SingleResourceTransportDto implements
    Comparable<VirtualMachineDto>
{
    private static final long serialVersionUID = -8877350185009627544L;

    public static final String BASE_MEDIA_TYPE = "application/vnd.abiquo.virtualmachine+xml";

    public static final String MEDIA_TYPE = BASE_MEDIA_TYPE + "; version=" + API_VERSION;

    public static final String MEDIA_TYPE_22 = BASE_MEDIA_TYPE + "; version=" + API_VERSION_22;

    public static final String MEDIA_TYPE_23 = BASE_MEDIA_TYPE + "; version=" + API_VERSION_23;

    private Integer id;

    private String name;

    private String description;

    private Integer ram;

    private Integer cpu;

    private Long hdInBytes;

    private int vdrpPort;

    private String vdrpIP;

    private int idState;

    private VirtualMachineState state;

    private int highDisponibility;

    private int idType;

    private String password;

    private String keymap;

    private String uuid;

    private RunlistElementsDto runlistElements;

    private DvdManagementDto dvd;

    @Override
    public int compareTo(final VirtualMachineDto dto)
    {
        VirtualMachineDto vm2 = dto;
        if (StringUtils.isNotBlank(this.getName()) && StringUtils.isNotBlank(vm2.getName()))
        {
            return this.getName().compareTo(vm2.getName());
        }
        else if (!StringUtils.isNotBlank(this.getName()) && !StringUtils.isNotBlank(vm2.getName()))
        {
            return 0;
        }
        else if (!StringUtils.isNotBlank(this.getName()) && StringUtils.isNotBlank(vm2.getName()))
        {
            return -1;
        }
        else
        {
            return 1;
        }
    }

    @Override
    public String getBaseMediaType()
    {
        return BASE_MEDIA_TYPE;
    }

    /**
     * Number of CPUs of the virtual machine.
     */
    @NotNull
    public Integer getCpu()
    {
        return cpu;
    }

    public String getDescription()
    {
        return description;
    }

    /**
     * Whether the virtual machine must mount a CD or DVD drive.
     */
    @XmlElement(name = "dvd")
    public DvdManagementDto getDvd()
    {
        return dvd;
    }

    /**
     * Hard disk size in Bytes.
     */
    @NotNull
    public Long getHdInBytes()
    {
        return hdInBytes;
    }

    /**
     * Deprecated
     */
    @NotNull
    public int getHighDisponibility()
    {
        return highDisponibility;
    }

    /**
     * Opaque ID of the virtual machine.
     * 
     * @output
     */
    public Integer getId()
    {
        return id;
    }

    /**
     * Opaque ID of the state.
     */
    public int getIdState()
    {
        return idState;
    }

    /**
     * Opaque ID of the virtual machine type.
     */
    public int getIdType()
    {
        return idType;
    }

    /**
     * Keyboard keymap to be setted on the virtual machine.
     */
    public String getKeymap()
    {
        return keymap;
    }

    @Override
    public String getMediaType()
    {
        return VirtualMachineDto.MEDIA_TYPE;
    }

    /**
     * Name of the virtual machine.
     */
    @NotNull
    public String getName()
    {
        return name;
    }

    /**
     * VNC password.
     */
    public String getPassword()
    {
        return password;
    }

    /**
     * Ram size of the virtual machine.
     */
    @NotNull
    public Integer getRam()
    {
        return ram;
    }

    /**
     * Chef runlist to bootstrap.
     */
    public RunlistElementsDto getRunlist()
    {
        return runlistElements;
    }

    /**
     * Current virtual machine state.
     */
    public VirtualMachineState getState()
    {
        return state;
    }

    /**
     * UUID of the virtual machine.
     * 
     * @output
     */
    public String getUuid()
    {
        return uuid;
    }

    /**
     * Virtual remote desktop IP.
     * 
     * @output
     */
    public String getVdrpIP()
    {
        return vdrpIP;
    }

    /**
     * Virtual remote desktop port.
     * 
     * @output
     */
    public int getVdrpPort()
    {
        return vdrpPort;
    }

    public void setCpu(final Integer cpu)
    {
        this.cpu = cpu;
    }

    public void setDescription(final String description)
    {
        this.description = description;
    }

    public void setDvd(final DvdManagementDto dvd)
    {
        this.dvd = dvd;
    }

    public void setHdInBytes(final Long hdInBytes)
    {
        this.hdInBytes = hdInBytes;
    }

    public void setHighDisponibility(final int highDisponibility)
    {
        this.highDisponibility = highDisponibility;
    }

    public void setId(final Integer id)
    {
        this.id = id;
    }

    public void setIdState(final int idState)
    {
        this.idState = idState;
    }

    public void setIdType(final int idType)
    {
        this.idType = idType;
    }

    public void setKeymap(final String keymap)
    {
        this.keymap = keymap;
    }

    public void setName(final String name)
    {
        this.name = name;
    }

    public void setPassword(final String password)
    {
        this.password = password;
    }

    public void setRam(final Integer ram)
    {
        this.ram = ram;
    }

    public void setRunlist(final RunlistElementsDto runlistElements)
    {
        this.runlistElements = runlistElements;
    }

    public void setState(final VirtualMachineState state)
    {
        this.state = state;
    }

    public void setUuid(final String uuid)
    {
        this.uuid = uuid;
    }

    public void setVdrpIP(final String vdrpIP)
    {
        this.vdrpIP = vdrpIP;
    }

    public void setVdrpPort(final int vdrpPort)
    {
        this.vdrpPort = vdrpPort;
    }

    /**
     * Modify the NIC links.
     * 
     * @param nics new NIC links
     */
    public void setNicLinks(final List<RESTLink> nics)
    {
        // First delete current Nic links. Convert to iterator
        // because it is impossible to delete objects in a list.
        Iterator<RESTLink> currentNics = getLinks().iterator();
        while (currentNics.hasNext())
        {
            if (currentNics.next().getRel().toLowerCase().startsWith("nic"))
            {
                currentNics.remove();
            }
        }

        for (RESTLink nicLink : nics)
        {
            // add nic link to list of links
            if (nicLink.getRel().toLowerCase().startsWith("nic"))
            {
                // avoid problems with another kind of rels
                getLinks().add(nicLink);
            }
        }
    }

}
