/**
 * Licensed to Abiquo Holdings S.L. (Abiquo) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package com.abiquo.server.core.enterprise;

import javax.validation.constraints.NotNull;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

import com.abiquo.model.transport.SingleResourceWithLimitsDto;

/**
 * The Enterprise entity offers the functionality of managing the enterprise infrastructure in a
 * logical way.
 * 
 * @author destevezg
 */
@XmlRootElement(name = "enterprise")
@XmlType(propOrder = {"id", "name", "chefURL", "chefClient", "chefClientCertificate",
"chefValidator", "chefValidatorCertificate", "repositoryHard", "repositorySoft",
"isReservationRestricted", "idPricingTemplate"})
public class EnterpriseDto extends SingleResourceWithLimitsDto
{
    private static final long serialVersionUID = 1L;

    public static final String BASE_MEDIA_TYPE = "application/vnd.abiquo.enterprise+xml";

    public static final String MEDIA_TYPE = BASE_MEDIA_TYPE + "; version=" + API_VERSION;

    private Integer id;

    private String name;

    private long repositorySoft;

    private long repositoryHard;

    private String chefURL;

    private String chefClient;

    private String chefValidator;

    private String chefClientCertificate;

    private String chefValidatorCertificate;

    private boolean isReservationRestricted;

    private Integer idPricingTemplate;

    /**
     * Identifier of the Entity
     * 
     * @output
     * @output
     */
    public Integer getId()
    {
        return id;
    }

    public void setId(final Integer id)
    {
        this.id = id;
    }

    /**
     * The name of the Enterprise
     * 
     * @output
     */
    @NotNull
    public String getName()
    {
        return name;
    }

    public void setName(final String name)
    {
        this.name = name;
    }

    /**
     * Soft limit stablished for repository size
     * 
     * @output
     */
    @NotNull
    public long getRepositorySoft()
    {
        return repositorySoft;
    }

    public void setRepositorySoft(final long repositorySoft)
    {
        this.repositorySoft = repositorySoft;
    }

    /**
     * Hard limit established for repository size
     * 
     * @output
     */
    @NotNull
    public long getRepositoryHard()
    {
        return repositoryHard;
    }

    public void setRepositoryHard(final long repositoryHard)
    {
        this.repositoryHard = repositoryHard;
    }

    public void setRepositoryLimits(final long soft, final long hard)
    {
        this.repositorySoft = soft;
        this.repositoryHard = hard;
    }

    /**
     * URL of the Chef Server containing recipes
     * 
     * @output
     */
    public String getChefURL()
    {
        return chefURL;
    }

    public void setChefURL(final String chefURL)
    {
        this.chefURL = chefURL;
    }

    /**
     * This must be a Chef admin client on the chef server. The Abiquo Server will use it to work
     * with the Chef Server
     * 
     * @output
     */
    public String getChefClient()
    {
        return chefClient;
    }

    public void setChefClient(final String chefClient)
    {
        this.chefClient = chefClient;
    }

    /**
     * The validator client is used to create nodes. You must use the name of the validator client
     * on the chef server
     * 
     * @output
     */
    public String getChefValidator()
    {
        return chefValidator;
    }

    public void setChefValidator(final String chefValidator)
    {
        this.chefValidator = chefValidator;
    }

    /**
     * The admin client certificate, which may be stored in a file called adminclient.pem
     * 
     * @output
     */
    public String getChefClientCertificate()
    {
        return chefClientCertificate;
    }

    public void setChefClientCertificate(final String chefClientCertificate)
    {
        this.chefClientCertificate = chefClientCertificate;
    }

    /**
     * The validator certificate, which may be stored in a file called organization-validator or
     * validation.pem
     * 
     * @output
     */

    public String getChefValidatorCertificate()
    {
        return chefValidatorCertificate;
    }

    public void setChefValidatorCertificate(final String chefValidatorCertificate)
    {
        this.chefValidatorCertificate = chefValidatorCertificate;
    }

    /**
     * You can also restrict an enterprise so that it may only deploy virtual machines on the
     * physical machines reserved for it and not on any others
     * 
     * @output
     */

    public boolean getIsReservationRestricted()
    {
        return isReservationRestricted;
    }

    public void setIsReservationRestricted(final boolean isReservationRestricted)
    {
        this.isReservationRestricted = isReservationRestricted;
    }

    /**
     * Identifier for the pricing template associated to this enterprise
     * 
     * @output
     */
    public Integer getIdPricingTemplate()
    {
        return idPricingTemplate;
    }

    public void setIdPricingTemplate(final Integer idPricingTemplate)
    {
        this.idPricingTemplate = idPricingTemplate;
    }

    @Override
    public String getMediaType()
    {
        return EnterpriseDto.MEDIA_TYPE;
    }

    @Override
    public String getBaseMediaType()
    {
        return BASE_MEDIA_TYPE;
    }

}
