/**
 * Licensed to Abiquo Holdings S.L. (Abiquo) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package com.abiquo.server.core.statistics;

import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

import com.abiquo.model.transport.SingleResourceTransportDto;

/**
 * Shows statistics data on cloud usage filtered by datacenter such as:
 * <ol>
 * <li>Total number of enterprises</li>
 * <li>Total number of users</li>
 * <li>Total number of virtual datacenters</li>
 * <li>Total number of physical machines managed running or stopped</li>
 * <li>Total virtual machines managed: running / not running</li>
 * <li>External storage (in volumes defined in datacenter's storage section) used by virtual
 * machines: limits / total / currently running</li>
 * <li>Limits virtual CPU / Total virtual CPU including oversubscription rules effect / Used virtual
 * CPU (red)</li>
 * <li>Hard disk resources: Limits (yellow) / Total hard disk resources (green) / Used hard disk
 * resources (red)</li>
 * <li>RAM in virtual machines: Limits by enterprises / Total memory including oversubscription
 * rules effect / Used memory</li>
 * <li>Total enterprise limits for public IPs / Total public IPs created and available in all
 * datacenters / Total public IPs used (reserved or used by virtual machines)</li>
 * <li>Total number of VLANs in use by any datacenter managed</li>
 * </ol>
 * 
 * @author destevezg
 */
@XmlRootElement(name = "cloudusage")
@XmlType(propOrder = {"id", "numEnterprisesCreated", "numUsersCreated", "numVdcCreated",
"publicIPsReserved", "publicIPsTotal", "publicIPsUsed", "serversRunning", "serversTotal",
"storageReserved", "storageTotal", "storageUsed", "virtualCpuReserved", "virtualCpuTotal",
"virtualCpuUsed", "virtualMachinesRunning", "virtualMachinesTotal", "virtualMemoryReserved",
"virtualMemoryTotal", "virtualMemoryUsed", "virtualStorageReserved", "virtualStorageTotal",
"virtualStorageUsed", "vlanUsed"})
public class CloudUsageDto extends SingleResourceTransportDto
{
    public static final String BASE_MEDIA_TYPE = "application/vnd.abiquo.cloudusage+xml";

    public static final String MEDIA_TYPE = BASE_MEDIA_TYPE + "; version=" + API_VERSION;

    public static final String MEDIA_TYPE_23 = BASE_MEDIA_TYPE + "; version=" + API_VERSION_23;

    public static final String MEDIA_TYPE_22 = BASE_MEDIA_TYPE + "; version=" + API_VERSION_22;

    private Integer id;

    /**
     * Identifier of the Entity
     * 
     * @output
     */
    public Integer getId()
    {
        return id;
    }

    public void setId(final Integer id)
    {
        this.id = id;
    }

    private long virtualStorageTotal;

    /**
     * Total storage size managed to be used by Virtual Machines in Megabytes
     * 
     * @output
     */
    public long getVirtualStorageTotal()
    {
        return virtualStorageTotal;
    }

    public void setVirtualStorageTotal(final long virtualStorageTotal)
    {
        this.virtualStorageTotal = virtualStorageTotal;
    }

    private long virtualMemoryReserved;

    /**
     * Maximum amount of RAM reserved by all enterprises in Megabytes
     * 
     * @output
     */
    public long getVirtualMemoryReserved()
    {
        return virtualMemoryReserved;
    }

    public void setVirtualMemoryReserved(final long virtualMemoryReserved)
    {
        this.virtualMemoryReserved = virtualMemoryReserved;
    }

    private long storageTotal;

    /**
     * External storage managed by Abiquo (in Megabytes)
     * 
     * @output
     */
    public long getStorageTotal()
    {
        return storageTotal;
    }

    public void setStorageTotal(final long storageTotal)
    {
        this.storageTotal = storageTotal;
    }

    private long virtualMemoryUsed;

    /**
     * Total amount of RAM used by Virtual Machines currently in a running state in Megabytes
     * 
     * @output
     */
    public long getVirtualMemoryUsed()
    {
        return virtualMemoryUsed;
    }

    public void setVirtualMemoryUsed(final long virtualMemoryUsed)
    {
        this.virtualMemoryUsed = virtualMemoryUsed;
    }

    private long virtualCpuTotal;

    /**
     * Total number of Virtual CPUs in Virtual Machines currently in a running state
     * 
     * @output
     */
    public long getVirtualCpuTotal()
    {
        return virtualCpuTotal;
    }

    public void setVirtualCpuTotal(final long virtualCpuTotal)
    {
        this.virtualCpuTotal = virtualCpuTotal;
    }

    private long publicIPsUsed;

    /**
     * Total number of IPs used by virtual machines
     * 
     * @output
     */
    public long getPublicIPsUsed()
    {
        return publicIPsUsed;
    }

    public void setPublicIPsUsed(final long publicIPsUsed)
    {
        this.publicIPsUsed = publicIPsUsed;
    }

    private long publicIPsReserved;

    /**
     * Total number of IPs assigned to a VirtualDatacenter (Reserved)
     * 
     * @output
     */
    public long getPublicIPsReserved()
    {
        return publicIPsReserved;
    }

    public void setPublicIPsReserved(final long publicIPsReserved)
    {
        this.publicIPsReserved = publicIPsReserved;
    }

    private long storageUsed;

    /**
     * External storage used (attached or mounted) by any virtual machines (in Megabytes)
     * 
     * @output
     */
    public long getStorageUsed()
    {
        return storageUsed;
    }

    public void setStorageUsed(final long storageUsed)
    {
        this.storageUsed = storageUsed;
    }

    private long storageReserved;

    /**
     * External storage reserved by Enterprises in Datacenters (in Megabytes)
     * 
     * @output
     */
    public long getStorageReserved()
    {
        return storageReserved;
    }

    public void setStorageReserved(final long storageReserved)
    {
        this.storageReserved = storageReserved;
    }

    private long serversTotal;

    /**
     * Total number of Physical machines managed
     * 
     * @output
     */
    public long getServersTotal()
    {
        return serversTotal;
    }

    public void setServersTotal(final long serversTotal)
    {
        this.serversTotal = serversTotal;
    }

    private long publicIPsTotal;

    /**
     * Total number of IPs defined (managed)
     * 
     * @output
     */
    public long getPublicIPsTotal()
    {
        return publicIPsTotal;
    }

    public void setPublicIPsTotal(final long publicIPsTotal)
    {
        this.publicIPsTotal = publicIPsTotal;
    }

    private long virtualStorageUsed;

    /**
     * Total storage size used by Virtual Machines currently in a running state in Megabytes
     * 
     * @output
     */
    public long getVirtualStorageUsed()
    {
        return virtualStorageUsed;
    }

    public void setVirtualStorageUsed(final long virtualStorageUsed)
    {
        this.virtualStorageUsed = virtualStorageUsed;
    }

    private long vlanUsed;

    /**
     * Total number of VLANs in use by any datacenter managed
     * 
     * @output
     */
    public long getVlanUsed()
    {
        return vlanUsed;
    }

    public void setVlanUsed(final long vlanUsed)
    {
        this.vlanUsed = vlanUsed;
    }

    private long numVdcCreated;

    /**
     * Total number of Virtual DataCenters in system
     * 
     * @output
     */
    public long getNumVdcCreated()
    {
        return numVdcCreated;
    }

    public void setNumVdcCreated(final long numVdcCreated)
    {
        this.numVdcCreated = numVdcCreated;
    }

    private long numEnterprisesCreated;

    /**
     * Total number of enterprises in system
     * 
     * @output
     */
    public long getNumEnterprisesCreated()
    {
        return numEnterprisesCreated;
    }

    public void setNumEnterprisesCreated(final long numEnterprisesCreated)
    {
        this.numEnterprisesCreated = numEnterprisesCreated;
    }

    private long virtualStorageReserved;

    /**
     * Maximum storage size reserved by all enterprises for Virtual Machines in Megabytes
     * 
     * @output
     */
    public long getVirtualStorageReserved()
    {
        return virtualStorageReserved;
    }

    public void setVirtualStorageReserved(final long virtualStorageReserved)
    {
        this.virtualStorageReserved = virtualStorageReserved;
    }

    private long virtualMemoryTotal;

    /**
     * Total amount of RAM managed (can be used for VMs) in Megabytes
     * 
     * @output
     */
    public long getVirtualMemoryTotal()
    {
        return virtualMemoryTotal;
    }

    public void setVirtualMemoryTotal(final long virtualMemoryTotal)
    {
        this.virtualMemoryTotal = virtualMemoryTotal;
    }

    private long virtualCpuUsed;

    /**
     * Total number of Virtual CPUs in Virtual Machines currently in a running state
     * 
     * @output
     */
    public long getVirtualCpuUsed()
    {
        return virtualCpuUsed;
    }

    public void setVirtualCpuUsed(final long virtualCpuUsed)
    {
        this.virtualCpuUsed = virtualCpuUsed;
    }

    private long virtualCpuReserved;

    /**
     * Maximum number of Virtual CPUs reserved by all enterprises
     * 
     * @output
     */
    public long getVirtualCpuReserved()
    {
        return virtualCpuReserved;
    }

    public void setVirtualCpuReserved(final long virtualCpuReserved)
    {
        this.virtualCpuReserved = virtualCpuReserved;
    }

    private long virtualMachinesTotal;

    /**
     * Total number of Virtual Machines managed
     * 
     * @output
     */
    public long getVirtualMachinesTotal()
    {
        return virtualMachinesTotal;
    }

    public void setVirtualMachinesTotal(final long virtualMachinesTotal)
    {
        this.virtualMachinesTotal = virtualMachinesTotal;
    }

    private long serversRunning;

    /**
     * Total number of Physical Machines running currently
     * 
     * @output
     */
    public long getServersRunning()
    {
        return serversRunning;
    }

    public void setServersRunning(final long serversRunning)
    {
        this.serversRunning = serversRunning;
    }

    private long numUsersCreated;

    /**
     * Total number of users in system
     * 
     * @output
     */
    public long getNumUsersCreated()
    {
        return numUsersCreated;
    }

    public void setNumUsersCreated(final long numUsersCreated)
    {
        this.numUsersCreated = numUsersCreated;
    }

    private long virtualMachinesRunning;

    /**
     * Total number of Virtual Machines currently in a running state
     * 
     * @output
     */
    public long getVirtualMachinesRunning()
    {
        return virtualMachinesRunning;
    }

    public void setVirtualMachinesRunning(final long virtualMachinesRunning)
    {
        this.virtualMachinesRunning = virtualMachinesRunning;
    }

    @Override
    public String getMediaType()
    {
        return CloudUsageDto.MEDIA_TYPE;
    }

    @Override
    public String getBaseMediaType()
    {
        return BASE_MEDIA_TYPE;
    }

}
