/**
 * Licensed to Abiquo Holdings S.L. (Abiquo) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package com.abiquo.server.core.statistics;

import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

import com.abiquo.model.transport.SingleResourceTransportDto;

/**
 * Shows statistical data on resources used by enterprises, such as:
 * <ol>
 * <li>Virtual CPUs used by the virtual machines (total and currently running)</li>
 * <li>Storage size (total and used) of disks used by virtual machines (total and currently running)
 * </li>
 * <li>RAM used by the virtual machines (total and currently running)</li>
 * <li>External storage (in volumes defined in the datacenter's storage section) used by virtual
 * machines (total and currently running)</li>
 * <li>Total and used number of VLANs defined</li>
 * <li>Total, reserved and used number of public IPs</li>
 * </ol>
 * 
 * @author destevezg
 */
@XmlRootElement(name = "enterpriseresources")
@XmlType(propOrder = {"id", "extStorageReserved", "extStorageUsed", "localStorageReserved",
"localStorageUsed", "memoryReserved", "memoryUsed", "publicIPsLimit", "publicIPsReserved",
"publicIPsUsed", "repositoryReserved", "repositoryUsed", "virtualCpuReserved", "virtualCpuUsed",
"vlanReserved", "vlanUsed"})
public class EnterpriseResourcesDto extends SingleResourceTransportDto
{
    public static final String BASE_MEDIA_TYPE = "application/vnd.abiquo.enterpriseresources+xml";

    public static final String MEDIA_TYPE = BASE_MEDIA_TYPE + "; version=" + API_VERSION;

    public static final String MEDIA_TYPE_23 = BASE_MEDIA_TYPE + "; version=" + API_VERSION_23;

    public static final String MEDIA_TYPE_22 = BASE_MEDIA_TYPE + "; version=" + API_VERSION_22;

    private Integer id;

    /**
     * Identifier of the Entity
     * 
     * @output
     */
    public Integer getId()
    {
        return id;
    }

    public void setId(final Integer id)
    {
        this.id = id;
    }

    private long repositoryReserved;

    /**
     * Maximum repository size reserved for this enterprise in Megabytes
     * 
     * @output
     */
    public long getRepositoryReserved()
    {
        return repositoryReserved;
    }

    public void setRepositoryReserved(final long repositoryReserved)
    {
        this.repositoryReserved = repositoryReserved;
    }

    private long memoryReserved;

    /**
     * Maximum amount of RAM reserved for this enterprise in Megabytes
     * 
     * @output
     */
    public long getMemoryReserved()
    {
        return memoryReserved;
    }

    public void setMemoryReserved(final long memoryReserved)
    {
        this.memoryReserved = memoryReserved;
    }

    private long vlanReserved;

    /**
     * Total maximum number of VLANs reserved for this enterprise. (enterprise.vlanHard)
     * 
     * @output
     */
    public long getVlanReserved()
    {
        return vlanReserved;
    }

    public void setVlanReserved(final long vlanReserved)
    {
        this.vlanReserved = vlanReserved;
    }

    private long publicIPsUsed;

    /**
     * Total number of IPs used by virtual machines
     * 
     * @output
     */
    public long getPublicIPsUsed()
    {
        return publicIPsUsed;
    }

    public void setPublicIPsUsed(final long publicIPsUsed)
    {
        this.publicIPsUsed = publicIPsUsed;
    }

    private long publicIPsReserved;

    /**
     * Total number of IPs assigned to a VirtualDatacenter (Reserved)
     * 
     * @output
     */
    public long getPublicIPsReserved()
    {
        return publicIPsReserved;
    }

    public void setPublicIPsReserved(final long publicIPsReserved)
    {
        this.publicIPsReserved = publicIPsReserved;
    }

    private long publicIPsLimit;

    /**
     * Maximum amount of public ips this enterprise can reserve
     * 
     * @param publicIPsLimit
     */

    public void setPublicIPsLimit(final long publicIPsLimit)
    {
        this.publicIPsLimit = publicIPsLimit;
    }

    public long getPublicIPsLimit()
    {
        return publicIPsLimit;
    }

    private long localStorageReserved;

    /**
     * Maximum storage size reserved for this enterprise for Virtual Machines in Megabytes
     * 
     * @output
     */

    public long getLocalStorageReserved()
    {
        return localStorageReserved;
    }

    public void setLocalStorageReserved(final long localStorageReserved)
    {
        this.localStorageReserved = localStorageReserved;
    }

    private long extStorageUsed;

    /**
     * Storage in external storage volumes used (attached or mounted) by any virtual machines (in
     * Megabytes)
     * 
     * @output
     */
    public long getExtStorageUsed()
    {
        return extStorageUsed;
    }

    public void setExtStorageUsed(final long extStorageUsed)
    {
        this.extStorageUsed = extStorageUsed;
    }

    private long vlanUsed;

    /**
     * Total number of VLANs in use by any datacenter managed
     * 
     * @output
     */

    public long getVlanUsed()
    {
        return vlanUsed;
    }

    public void setVlanUsed(final long vlanUsed)
    {
        this.vlanUsed = vlanUsed;
    }

    private long extStorageReserved;

    /**
     * Maximum external storage size reserved for this enterprise in Megabytes
     * 
     * @output
     */

    public long getExtStorageReserved()
    {
        return extStorageReserved;
    }

    public void setExtStorageReserved(final long extStorageReserved)
    {
        this.extStorageReserved = extStorageReserved;
    }

    private long localStorageUsed;

    /**
     * Total storage size used by Virtual Machines currently in a running state in Megabytes
     * 
     * @output
     */

    public long getLocalStorageUsed()
    {
        return localStorageUsed;
    }

    public void setLocalStorageUsed(final long localStorageUsed)
    {
        this.localStorageUsed = localStorageUsed;
    }

    private long virtualCpuUsed;

    /**
     * Total number of Virtual CPUs in Virtual Machines currently in a running state
     * 
     * @output
     */

    public long getVirtualCpuUsed()
    {
        return virtualCpuUsed;
    }

    public void setVirtualCpuUsed(final long virtualCpuUsed)
    {
        this.virtualCpuUsed = virtualCpuUsed;
    }

    private long virtualCpuReserved;

    /**
     * Maximum number of Virtual CPUs reserved for this enterprise
     * 
     * @output
     */

    public long getVirtualCpuReserved()
    {
        return virtualCpuReserved;
    }

    public void setVirtualCpuReserved(final long virtualCpuReserved)
    {
        this.virtualCpuReserved = virtualCpuReserved;
    }

    private long repositoryUsed;

    /**
     * Total repository size currently used for this enterprise in Megabytes
     * 
     * @output
     */

    public long getRepositoryUsed()
    {
        return repositoryUsed;
    }

    public void setRepositoryUsed(final long repositoryUsed)
    {
        this.repositoryUsed = repositoryUsed;
    }

    private long memoryUsed;

    /**
     * Total amount of RAM used by Virtual Machines currently in a running state in Megabytes
     * 
     * @output
     */
    public long getMemoryUsed()
    {
        return memoryUsed;
    }

    public void setMemoryUsed(final long memoryUsed)
    {
        this.memoryUsed = memoryUsed;
    }

    @Override
    public String getMediaType()
    {
        return EnterpriseResourcesDto.MEDIA_TYPE;
    }

    @Override
    public String getBaseMediaType()
    {
        return BASE_MEDIA_TYPE;
    }

}
